@@ -1,32 +1,43 @@
 """Codex Django Models."""
+import calendar
 import datetime
+import os
 
-from logging import getLogger
+from decimal import Decimal
 from pathlib import Path
+from uuid import uuid4
 
 from django.conf import settings
+from django.contrib.auth.models import Group
 from django.contrib.sessions.models import Session
 from django.core.exceptions import ValidationError
-from django.db.models import CASCADE
-from django.db.models import SET
-from django.db.models import BooleanField
-from django.db.models import CharField
-from django.db.models import DateField
-from django.db.models import DateTimeField
-from django.db.models import DecimalField
-from django.db.models import DurationField
-from django.db.models import ForeignKey
-from django.db.models import ManyToManyField
-from django.db.models import Model
-from django.db.models import PositiveSmallIntegerField
-from django.db.models import TextField
-from django.db.models import URLField
+from django.db import connection
+from django.db.models import (
+    CASCADE,
+    BooleanField,
+    CharField,
+    DateField,
+    DateTimeField,
+    DecimalField,
+    DurationField,
+    ForeignKey,
+    Index,
+    JSONField,
+    ManyToManyField,
+    Model,
+    PositiveIntegerField,
+    PositiveSmallIntegerField,
+    TextField,
+    URLField,
+)
 from django.utils.translation import gettext_lazy as _
 
-from codex.serializers.webpack import CHOICES
+from codex.serializers.choices import CHOICES
+from codex.settings.logging import get_logger
+from codex.settings.settings import XAPIAN_INDEX_PATH, XAPIAN_INDEX_UUID_PATH
 
 
-LOG = getLogger(__name__)
+LOG = get_logger(__name__)
 
 
 SCHEMA_VERSION = 1
@@ -43,99 +54,87 @@ class BaseModel(Model):
         """Without this a real table is created and joined to."""
 
         abstract = True
+        get_latest_by = "updated_at"
 
 
 class BrowserGroupModel(BaseModel):
     """Browser groups."""
 
-    is_default = BooleanField(default=False)
-    sort_name = CharField(db_index=True, max_length=32)
+    DEFAULT_NAME = ""
+
+    name = CharField(db_index=True, max_length=64, default=DEFAULT_NAME)
+    sort_name = CharField(db_index=True, max_length=130, default=DEFAULT_NAME)
+
+    def presave(self):
+        """Save the sort name. Called by save()."""
+        self.sort_name = self.name
+
+    def save(self, *args, **kwargs):
+        """Save the sort name as the name by default."""
+        self.presave()
+        super().save(*args, **kwargs)
 
     class Meta:
         """Without this a real table is created and joined to."""
 
         abstract = True
+        ordering = ["sort_name"]
 
 
 class Publisher(BrowserGroupModel):
     """The publisher of the comic."""
 
-    DEFAULTS = {"is_default": True}
-
-    name = CharField(max_length=32, default="No Publisher")
-
-    @classmethod
-    def get_default_publisher(cls):
-        """Get or create a default 'No Publisher' entry."""
-        publisher, _ = cls.objects.get_or_create(defaults=cls.DEFAULTS, **cls.DEFAULTS)
-        return publisher
-
-    def save(self, *args, **kwargs):
-        """Save the sort name as the name by default."""
-        self.sort_name = self.name
-        super().save(*args, **kwargs)
-
     class Meta:
         """Constraints."""
 
-        unique_together = ("name", "is_default")
+        unique_together = ("name",)
 
 
 class Imprint(BrowserGroupModel):
     """A Publishing imprint."""
 
-    name = CharField(max_length=32, default="Main Imprint")
-    publisher = ForeignKey(Publisher, on_delete=SET(Publisher.get_default_publisher))
+    publisher = ForeignKey(Publisher, on_delete=CASCADE)
 
     class Meta:
-        """Contraints."""
+        """Constraints."""
 
-        unique_together = ("name", "publisher", "is_default")
+        unique_together = ("name", "publisher")
 
-    def save(self, *args, **kwargs):
-        """Save the sort name."""
+    def presave(self):
+        """Save the sort name. Called by save()."""
         self.sort_name = f"{self.publisher.name} {self.name}"
-        super().save(*args, **kwargs)
 
 
 class Series(BrowserGroupModel):
     """The series the comic belongs to."""
 
-    name = CharField(max_length=32, default="Default Series")
-    publisher = ForeignKey(Publisher, on_delete=Publisher.get_default_publisher)
+    publisher = ForeignKey(Publisher, on_delete=CASCADE)
     imprint = ForeignKey(Imprint, on_delete=CASCADE)
     volume_count = PositiveSmallIntegerField(null=True)
 
-    def save(self, *args, **kwargs):
-        """Save the sort name as the name by default."""
-        self.sort_name = self.name
-        super().save(*args, **kwargs)
-
     class Meta:
         """Constraints."""
 
-        unique_together = ("name", "imprint", "is_default")
+        unique_together = ("name", "imprint")
         verbose_name_plural = "Series"
 
 
 class Volume(BrowserGroupModel):
     """The volume of the series the comic belongs to."""
 
-    name = CharField(max_length=32, default="")
-    publisher = ForeignKey(Publisher, on_delete=Publisher.get_default_publisher)
+    publisher = ForeignKey(Publisher, on_delete=CASCADE)
     imprint = ForeignKey(Imprint, on_delete=CASCADE)
     series = ForeignKey(Series, on_delete=CASCADE)
-    issue_count = DecimalField(decimal_places=2, max_digits=6, null=True)
+    issue_count = PositiveSmallIntegerField(null=True)
 
-    def save(self, *args, **kwargs):
-        """Save the sort name."""
+    def presave(self):
+        """Save the sort name. Called by save()."""
         self.sort_name = f"{self.series.name} {self.name}"
-        super().save(*args, **kwargs)
 
     class Meta:
         """Constraints."""
 
-        unique_together = ("name", "series", "is_default")
+        unique_together = ("name", "series")
 
 
 def validate_dir_exists(path):
@@ -147,17 +146,19 @@ def validate_dir_exists(path):
 class Library(BaseModel):
     """The library comic file live under."""
 
-    DEFAULT_SCAN_FREQUENCY = datetime.timedelta(seconds=12 * 60 * 60)
+    DEFAULT_POLL_EVERY_SECONDS = 60 * 60
+    DEFAULT_POLL_EVERY = datetime.timedelta(seconds=DEFAULT_POLL_EVERY_SECONDS)
 
     path = CharField(
         unique=True, db_index=True, max_length=128, validators=[validate_dir_exists]
     )
-    enable_watch = BooleanField(db_index=True, default=True)
-    enable_scan_cron = BooleanField(db_index=True, default=True)
-    scan_frequency = DurationField(default=DEFAULT_SCAN_FREQUENCY)
-    last_scan = DateTimeField(null=True)
-    scan_in_progress = BooleanField(default=False)
+    events = BooleanField(db_index=True, default=True)
+    poll = BooleanField(db_index=True, default=True)
+    poll_every = DurationField(default=DEFAULT_POLL_EVERY)
+    last_poll = DateTimeField(null=True)
+    update_in_progress = BooleanField(default=False)
     schema_version = PositiveSmallIntegerField(default=0)
+    groups = ManyToManyField(Group, blank=True)
 
     def __str__(self):
         """Return the path."""
@@ -172,10 +173,10 @@ class Library(BaseModel):
 class NamedModel(BaseModel):
     """A for simple named tables."""
 
-    name = CharField(db_index=True, max_length=32)
+    name = CharField(db_index=True, max_length=64)
 
     class Meta:
-        """Defaults to uniquely named, must be overriden."""
+        """Defaults to uniquely named, must be overridden."""
 
         abstract = True
         unique_together = ("name",)
@@ -185,33 +186,6 @@ class NamedModel(BaseModel):
         return self.name
 
 
-class Folder(NamedModel):
-    """File system folder."""
-
-    PARENT_FIELD = "folder"
-    path = CharField(max_length=128, db_index=True, validators=[validate_dir_exists])
-    library = ForeignKey(Library, on_delete=CASCADE)
-    parent_folder = ForeignKey(
-        "Folder",
-        on_delete=CASCADE,
-        null=True,
-    )
-    sort_name = CharField(max_length=32)
-
-    def save(self, *args, **kwargs):
-        """Save the sort name as the name by default."""
-        self.sort_name = self.name
-        super().save(*args, **kwargs)
-
-    class Meta:
-        """Constraints."""
-
-        unique_together = ("library", "path")
-
-
-Folder.CHILD_CLASS = Folder
-
-
 class SeriesGroup(NamedModel):
     """A series group the series is part of."""
 
@@ -276,47 +250,103 @@ class Credit(BaseModel):
         unique_together = ("person", "role")
 
 
-class Comic(BaseModel):
+class WatchedPath(BrowserGroupModel):
+    """A filesystem path with data for Watchdog."""
+
+    library = ForeignKey(Library, on_delete=CASCADE, db_index=True)
+    path = CharField(max_length=4095, db_index=True)
+    stat = JSONField(null=True)
+    parent_folder = ForeignKey(
+        "Folder",
+        on_delete=CASCADE,
+        null=True,
+    )
+    ZERO_STAT = [0, 0, 0, 0, 0, 0, 0, 0, 0.0, 0]
+
+    def set_stat(self):
+        """Set select stat params from the filesystem."""
+        st_record = os.stat(self.path)
+        # Converting os.stat directly to a list or tuple saves
+        # mtime as an int and causes problems.
+        st = self.ZERO_STAT.copy()
+        st[0] = st_record.st_mode
+        st[1] = st_record.st_ino
+        # st_dev changes every time with docker
+        # st[2] = st_record.st_dev
+        st[6] = st_record.st_size
+        st[8] = st_record.st_mtime
+        self.stat = st
+
+    def __str__(self):
+        """Return the full path."""
+        return self.path
+
+    class Meta:
+        """Constraints."""
+
+        unique_together = ("library", "path")
+        abstract = True
+
+
+class Folder(WatchedPath):
+    """File system folder."""
+
+
+class Comic(WatchedPath):
     """Comic metadata."""
 
-    path = CharField(db_index=True, max_length=128)
+    # From BaseModel, but Comics are sorted by these so index them
+    created_at = DateTimeField(auto_now_add=True, db_index=True)
+    updated_at = DateTimeField(auto_now=True, db_index=True)
+
+    # From WatchedPath, but interferes with related_name from folders m2m field
+    parent_folder = ForeignKey(
+        "Folder", on_delete=CASCADE, null=True, related_name="comic_in"
+    )
+
+    # Unique comic fields
+    issue = DecimalField(
+        db_index=True, decimal_places=2, max_digits=10, default=Decimal(0.0)
+    )
     volume = ForeignKey(Volume, db_index=True, on_delete=CASCADE)
     series = ForeignKey(Series, db_index=True, on_delete=CASCADE)
     imprint = ForeignKey(Imprint, db_index=True, on_delete=CASCADE)
     publisher = ForeignKey(Publisher, db_index=True, on_delete=CASCADE)
-    issue = DecimalField(db_index=True, decimal_places=2, max_digits=6, default=0.0)
-    title = CharField(db_index=True, max_length=64, null=True)
     # Date
-    year = PositiveSmallIntegerField(null=True)
+    year = PositiveSmallIntegerField(db_index=True, null=True)
     month = PositiveSmallIntegerField(null=True)
     day = PositiveSmallIntegerField(null=True)
     # Summary
-    summary = TextField(null=True)
+    comments = TextField(null=True)
     notes = TextField(null=True)
-    description = TextField(null=True)
+    summary = TextField(null=True)
     # Ratings
-    critical_rating = CharField(db_index=True, max_length=32, null=True)
-    maturity_rating = CharField(db_index=True, max_length=32, null=True)
-    user_rating = CharField(db_index=True, max_length=32, null=True)
+    community_rating = DecimalField(
+        db_index=True, decimal_places=2, max_digits=5, default=None, null=True
+    )
+    critical_rating = DecimalField(
+        db_index=True, decimal_places=2, max_digits=5, default=None, null=True
+    )
+    age_rating = CharField(db_index=True, max_length=32, null=True)
     # alpha2 fields for countries
     country = CharField(db_index=True, max_length=32, null=True)
-    language = CharField(db_index=True, max_length=16, null=True)
+    language = CharField(db_index=True, max_length=32, null=True)
     # misc
+    cover_image = CharField(max_length=256, null=True)
+    format = CharField(db_index=True, max_length=16, null=True)
     page_count = PositiveSmallIntegerField(db_index=True, default=0)
-    cover_image = CharField(max_length=64, null=True)
-    read_ltr = BooleanField(default=True)
+    read_ltr = BooleanField(db_index=True, default=True)
+    scan_info = CharField(max_length=128, null=True)
     web = URLField(null=True)
-    format = CharField(max_length=16, null=True)
-    scan_info = CharField(max_length=32, null=True)
     # ManyToMany
-    credits = ManyToManyField(Credit)
-    tags = ManyToManyField(Tag)
-    teams = ManyToManyField(Team)
     characters = ManyToManyField(Character)
+    credits = ManyToManyField(Credit)
+    genres = ManyToManyField(Genre)
     locations = ManyToManyField(Location)
     series_groups = ManyToManyField(SeriesGroup)
     story_arcs = ManyToManyField(StoryArc)
-    genres = ManyToManyField(Genre)
+    tags = ManyToManyField(Tag)
+    teams = ManyToManyField(Team)
     # Ignore these, they seem useless:
     #
     # black_and_white = BooleanField(default=False)
@@ -333,31 +363,36 @@ class Comic(BaseModel):
     # identifier = CharField(max_length=64, null=True)
 
     # codex only
-    library = ForeignKey(Library, on_delete=CASCADE, db_index=True)
-    sort_name = CharField(db_index=True, max_length=32)
+    cover_path = CharField(max_length=4095)
     date = DateField(db_index=True, null=True)
     decade = PositiveSmallIntegerField(db_index=True, null=True)
-    size = PositiveSmallIntegerField(db_index=True)
+    folders = ManyToManyField(Folder)
     max_page = PositiveSmallIntegerField(default=0)
-    parent_folder = ForeignKey(
-        Folder, db_index=True, on_delete=CASCADE, null=True, related_name="comic_in"
-    )
-    folder = ManyToManyField(Folder)
-    cover_path = CharField(max_length=32)
-    myself = ForeignKey("self", on_delete=CASCADE, null=True, related_name="comic")
+    size = PositiveIntegerField(db_index=True)
 
     class Meta:
         """Constraints."""
 
-        # prevents None path comics from being duplicated
-        unique_together = ("path", "volume", "year", "issue")
+        unique_together = ("library", "path")
         verbose_name = "Issue"
 
     def _set_date(self):
         """Compute a date for the comic."""
-        year = self.year if self.year is not None else datetime.MINYEAR
-        month = self.month if self.month is not None else 1
-        day = self.day if self.day is not None else 1
+        if self.year is None:
+            year = datetime.MINYEAR
+        else:
+            year = min(max(self.year, datetime.MINYEAR), datetime.MAXYEAR)
+        if self.month is None:
+            month = 1
+        else:
+            month = min(max(self.month, 1), 12)
+
+        if self.day is None:
+            day = 1
+        else:
+            last_day_of_month = calendar.monthrange(year, month)[1]
+            day = min(max(self.day, 1), last_day_of_month)
+
         self.date = datetime.date(year, month, day)
 
     def _set_decade(self):
@@ -367,6 +402,20 @@ class Comic(BaseModel):
         else:
             self.decade = self.year - (self.year % 10)
 
+    def presave(self):
+        """Set computed values."""
+        self._set_date()
+        self._set_decade()
+        self.max_page = max(self.page_count - 1, 0)
+        self.size = Path(self.path).stat().st_size
+        sort_names = (self.volume.sort_name, f"{self.issue:06.1f}")
+        self.sort_name = " ".join(sort_names)
+
+    def save(self, *args, **kwargs):
+        """Save computed fields."""
+        self.presave()
+        super().save(*args, **kwargs)
+
     def _get_display_issue(self):
         """Get the issue number, even if its a half issue."""
         if self.issue % 1 == 0:
@@ -375,12 +424,17 @@ class Comic(BaseModel):
             issue_str = f"#{self.issue:05.1f}"
         return issue_str
 
-    def save(self, *args, **kwargs):
-        """Save computed fields."""
-        self._set_date()
-        self._set_decade()
-        self.sort_name = f"{self.volume.sort_name} {self.issue:06.1f}"
-        super().save(*args, **kwargs)
+    def __str__(self):
+        """Most common text representation for logging."""
+        names = []
+        if self.series.name:
+            names.append(self.series.name)
+        if self.volume.name:
+            names.append(self.volume.name)
+        names.append(self._get_display_issue())
+        if self.name:
+            names.append(self.name)
+        return " ".join(names)
 
 
 class AdminFlag(NamedModel):
@@ -401,11 +455,11 @@ class AdminFlag(NamedModel):
     on = BooleanField(default=True)
 
 
-def cascade_if_user_null(collector, field, sub_objs, using):
+def cascade_if_user_null(collector, field, sub_objs, _using):
     """
     Cascade only if the user field is null.
 
-    Do this to keep deleteing ephemeral session data from UserBookmark table.
+    Do this to keep deleting ephemeral session data from UserBookmark table.
     Adapted from:
     https://github.com/django/django/blob/master/django/db/models/deletion.py#L23
     """
@@ -424,7 +478,7 @@ def cascade_if_user_null(collector, field, sub_objs, using):
             # fail_on_restricted=False,
         )
 
-    # Set them all to null, tho
+    # Set them all to null
     if field.null:
         # and not connections[using].features.can_defer_constraint_checks:
         collector.add_field_update(field, None, sub_objs)
@@ -459,28 +513,96 @@ class UserBookmark(BaseModel):
         unique_together = ("user", "session", "comic")
 
 
-class FailedImport(BaseModel):
+class FailedImport(WatchedPath):
     """Failed Comic Imports. Displayed in Admin Panel."""
 
-    MAX_REASON_LEN = 64
+    MAX_REASON_LEN = 32
 
-    library = ForeignKey(Library, db_index=True, on_delete=CASCADE)
-    path = CharField(db_index=True, max_length=128)
-    reason = CharField(max_length=MAX_REASON_LEN)
-
-    @classmethod
-    def get_reason(cls, exc, path):
+    def set_reason(self, exc):
         """Can't do this in save() because it breaks update_or_create."""
         reason = str(exc)
-        suffixes = (f": {path}", f": '{path}'")
+        suffixes = (f": {self.path}", f": '{self.path}'")
         for suffix in suffixes:
-            if reason.endswith(suffix):
-                reason = reason[: -len(suffix)]
-        reason = reason[: cls.MAX_REASON_LEN]
-        reason = reason.strip()
-        return reason
+            reason = reason.removesuffix(suffix)
+        reason = reason[: self.MAX_REASON_LEN]
+        self.name = reason.strip()
 
     class Meta:
         """Constraints."""
 
         unique_together = ("library", "path")
+
+
+class LatestVersion(BaseModel):
+    """Latest codex version."""
+
+    CODEX_VERSION_PK = 1
+    XAPIAN_INDEX_VERSION_PK = 2
+    version = CharField(max_length=32)
+
+    @classmethod
+    def _update_or_create(cls, pk, version):
+        search_kwargs = {"pk": pk}
+        defaults = {"version": version}
+        cls.objects.update_or_create(defaults=defaults, **search_kwargs)
+
+    @classmethod
+    def set_codex_version(cls, version):
+        """Ensure a single database row."""
+        cls._update_or_create(cls.CODEX_VERSION_PK, version)
+
+    @classmethod
+    def set_xapian_index_version(cls):
+        """Set the codex db to xapian matching id."""
+        version = str(uuid4())
+        cls._update_or_create(cls.XAPIAN_INDEX_VERSION_PK, version)
+        XAPIAN_INDEX_PATH.mkdir(parents=True, exist_ok=True)
+        with XAPIAN_INDEX_UUID_PATH.open("w") as uuid_file:
+            uuid_file.write(version)
+
+    @classmethod
+    def is_xapian_uuid_match(cls):
+        """Is this xapian index for this database."""
+        result = False
+        try:
+            with XAPIAN_INDEX_UUID_PATH.open("r") as uuid_file:
+                version = uuid_file.read()
+            lv = cls.objects.only("pk").get(
+                pk=cls.XAPIAN_INDEX_VERSION_PK, version=version
+            )
+            result = lv.pk == cls.XAPIAN_INDEX_VERSION_PK
+        except (FileNotFoundError, cls.DoesNotExist):
+            pass
+        except Exception as exc:
+            LOG.exception(exc)
+        return result
+
+
+class SearchQuery(Model):
+    """Search queries."""
+
+    text = CharField(db_index=True, unique=True, max_length=256)
+    used_at = DateTimeField(auto_now_add=True, db_index=True)
+
+
+class SearchResult(Model):
+    """results model."""
+
+    query = ForeignKey(SearchQuery, on_delete=CASCADE)
+    comic = ForeignKey(Comic, on_delete=CASCADE)
+    score = PositiveSmallIntegerField()
+
+    @classmethod
+    def truncate_and_reset(cls):
+        """Nuke this table and reset the autoincrementer."""
+        cls.objects.all().delete()
+        with connection.cursor() as cursor:
+            cursor.execute(
+                "UPDATE sqlite_sequence SET seq=1 WHERE name=%s", [cls._meta.db_table]
+            )
+
+    class Meta:
+        """These are usually looked up by comic & autoquery hash."""
+
+        unique_together = ("query", "comic")
+        indexes = [Index(fields=["comic", "query"])]
