@@ -1,19 +1,21 @@
 """Serializers for the browser view."""
+from abc import ABC
+
 from django.core.exceptions import ValidationError
 from django.utils.translation import gettext_lazy as _
-from rest_framework.serializers import BooleanField
-from rest_framework.serializers import CharField
-from rest_framework.serializers import ChoiceField
-from rest_framework.serializers import DecimalField
-from rest_framework.serializers import IntegerField
-from rest_framework.serializers import ListField
-from rest_framework.serializers import Serializer
-from rest_framework.serializers import SerializerMethodField
+from rest_framework.serializers import (
+    BooleanField,
+    CharField,
+    ChoiceField,
+    DecimalField,
+    Field,
+    IntegerField,
+    ListField,
+    Serializer,
+)
 
-from codex.librarian.queue import QUEUE
-from codex.librarian.queue import ComicCoverCreateTask
-from codex.serializers.webpack import CHOICES
-from codex.serializers.webpack import VUETIFY_NULL_CODE
+from codex.serializers.choices import CHOICES, VUETIFY_NULL_CODE
+from codex.serializers.mixins import BrowserAggregateSerializerMixin
 
 
 VUETIFY_NULL_CODE_STR = str(VUETIFY_NULL_CODE)
@@ -69,20 +71,28 @@ class BrowserSettingsShowGroupFlagsSerializer(Serializer):
     v = BooleanField()
 
 
-class FilterListField(ListField):
+class FilterListField(ListField, ABC):
     """Filter List field with custom arguments."""
 
+    CHILD_CLASS = Field
+    VALIDATORS = tuple()
+
     def __init__(self, *args, **kwargs):
         """Apply the subclass's arguments."""
-        validators = self.VALIDATORS + kwargs.pop("validators", tuple())
         super().__init__(
             *args,
-            child=self.CHILD_CLASS(allow_null=True),
-            validators=validators,
-            **kwargs,
+            child=self.CHILD_CLASS(allow_null=True, **kwargs),
+            validators=self.VALIDATORS,
         )
 
 
+class DecimalListField(FilterListField):
+    """Decimal List Field with validation."""
+
+    CHILD_CLASS = DecimalField
+    VALIDATORS = (validate_int_null,)
+
+
 class IntListField(FilterListField):
     """Integer List Field with validation."""
 
@@ -90,7 +100,13 @@ class IntListField(FilterListField):
     VALIDATORS = (validate_int_null,)
 
 
-class CharListField(ListField):
+class DecadeListField(IntListField):
+    """Integer List Field with validation for decades."""
+
+    VALIDATORS = (validate_int_null, validate_decades)
+
+
+class CharListField(FilterListField):
     """Char List Field with validation."""
 
     CHILD_CLASS = CharField
@@ -104,22 +120,22 @@ class BrowserSettingsFilterSerializer(Serializer):
         choices=tuple(CHOICES["bookmarkFilter"].keys())
     )
     # Dynamic filters
+    community_rating = DecimalListField(max_digits=4, decimal_places=2)
     characters = IntListField()
     country = CharListField()
     creators = IntListField()
-    critical_rating = CharListField()
-    decade = IntListField(validators=(validate_decades,))
+    critical_rating = DecimalListField(max_digits=4, decimal_places=2)
+    decade = DecadeListField()
     format = CharListField()
     genres = IntListField()
     language = CharListField()
     locations = IntListField()
-    maturity_rating = CharListField()
+    age_rating = CharListField()
     read_ltr = ListField(child=BooleanField())
     series_groups = IntListField()
     story_arcs = IntListField()
     tags = IntListField()
     teams = IntListField()
-    user_rating = CharListField()
     year = IntListField()
 
 
@@ -132,35 +148,24 @@ class BrowserSettingsSerializer(Serializer):
     """
 
     filters = BrowserSettingsFilterSerializer()
-    rootGroup = ChoiceField(choices=tuple(CHOICES["rootGroup"].keys()))  # noqa: N815
-    sortBy = ChoiceField(choices=tuple(CHOICES["sort"].keys()))  # noqa: N815
-    sortReverse = BooleanField()  # noqa: N815
+    autoquery = CharField(allow_blank=True)
+    topGroup = ChoiceField(choices=tuple(CHOICES["topGroup"].keys()))  # noqa: N815
+    orderBy = ChoiceField(choices=tuple(CHOICES["orderBy"].keys()))  # noqa: N815
+    orderReverse = BooleanField()  # noqa: N815
     show = BrowserSettingsShowGroupFlagsSerializer()
 
 
-class BrowserCardSerializer(Serializer):
-    """Generic browse object."""
-
-    def get_x_cover_path(self, obj):
-        """Ensure comic cover exists for any cover_path we send."""
-        comic_path = obj.get("x_path")
-        cover_path = obj.get("x_cover_path")
-        task = ComicCoverCreateTask(comic_path, cover_path, False)
-        QUEUE.put(task)
-        return cover_path
+class BrowserCardSerializer(BrowserAggregateSerializerMixin, Serializer):
+    """Browse card displayed in the browser."""
 
     pk = IntegerField(read_only=True)
     group = CharField(read_only=True, max_length=1)
-    child_count = IntegerField(read_only=True, allow_null=True)
-    x_cover_path = SerializerMethodField()
-    header_name = CharField(read_only=True)
+    cover_path = CharField(read_only=True)
+    publisher_name = CharField(read_only=True)
     series_name = CharField(read_only=True)
     volume_name = CharField(read_only=True)
-    x_issue = DecimalField(max_digits=5, decimal_places=1, read_only=True)
-    display_name = CharField(read_only=True)
-    progress = DecimalField(read_only=True, max_digits=5, decimal_places=2)
-    finished = BooleanField(read_only=True, allow_null=True)
-    bookmark = IntegerField(read_only=True, allow_null=True)
+    name = CharField(read_only=True)
+    issue = DecimalField(max_digits=5, decimal_places=1, read_only=True)
     order_value = CharField(read_only=True)
 
 
@@ -189,7 +194,10 @@ class BrowserTitleSerializer(Serializer):
 class BrowserPageSerializer(Serializer):
     """The main browse list."""
 
+    NUM_AUTOCOMPLETE_QUERIES = 10
+
     browserTitle = BrowserTitleSerializer(read_only=True)  # noqa: N815
+    modelGroup = CharField(read_only=True)  # noqa: N815
     upRoute = BrowserRouteSerializer(allow_null=True)  # noqa: N815
     objList = ListField(  # noqa: N815
         child=BrowserCardSerializer(read_only=True),
@@ -199,6 +207,9 @@ class BrowserPageSerializer(Serializer):
     numPages = IntegerField(read_only=True)  # noqa: N815
     formChoices = BrowserFormChoicesSerializer(read_only=True)  # noqa: N815
     librariesExist = BooleanField(read_only=True)  # noqa: N815
+    queries = ListField(
+        child=CharField(read_only=True), allow_empty=True, read_only=True
+    )
 
 
 class VersionsSerializer(Serializer):
